package com.example.astronomy

import android.app.*
import android.graphics.*
import android.os.*
import android.widget.*
import androidx.activity.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import retrofit2.*

class MainActivity : ComponentActivity() {
  val coroutineScope = MainScope()

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(TextView(this).apply { text = "Loading your image…" })
    coroutineScope.launch {
      while (true) {
        loadImage()
        delay(15.seconds)
      }
    }
  }

  override fun onDestroy() {
    super.onDestroy()
    coroutineScope.cancel()
  }
}

suspend fun Activity.loadImage() {
  val infoRequest = astronomyService.getPictures(count = 1, apiKey)
  val info = infoRequest.awaitResponse().body()!!.single()
  val imageRequest = astronomyService.downloadImage(info.url)
  val image = imageRequest.awaitResponse().body()!!.bytes()

  val bitmap = BitmapFactory.decodeStream(image.inputStream())
  setContentView(ImageView(this).apply { setImageBitmap(bitmap) })
}
