package com.example.astronomy

import okhttp3.*
import retrofit2.*
import retrofit2.Call
import retrofit2.converter.moshi.*
import retrofit2.http.*
import java.util.concurrent.*

const val apiKey = "DEMO_KEY"

interface AstronomyService {
  @GET("apod")
  fun getPictures(
    @Query("count") count: Int,
    @Query("api_key") apiKey: String,
  ): Call<List<PictureInfo>>

  @GET
  fun downloadImage(@Url imageUrl: String): Call<ResponseBody>
}

data class PictureInfo(val title: String, val url: String)

private val executor = Executors.newCachedThreadPool { task ->
  Thread(task).apply { isDaemon = true }
}

private fun createHttpClient() = OkHttpClient.Builder()
  .protocols(listOf(Protocol.HTTP_1_1)).dispatcher(Dispatcher(executor))

val astronomyService = Retrofit.Builder()
  .client(createHttpClient().build())
  .baseUrl("https://api.nasa.gov/planetary/")
  .addConverterFactory(MoshiConverterFactory.create())
  .build().create<AstronomyService>()
