package com.example.astronomy

import javax.swing.*
import kotlinx.coroutines.*
import retrofit2.*

class ImageGallery : ImageViewer() {
  override suspend fun update(window: JFrame) = coroutineScope {
    val infoRequest = astronomyService.getPictures(count = 2, apiKey)
    val (info1, info2) = infoRequest.awaitResponse().body()!!

    val image1: Deferred<ByteArray> = async { downloadImage(info1.url) }
    val image2: Deferred<ByteArray> = async { downloadImage(info2.url) }

    displayImageGallery(window, listOf(image1.await(), image2.await()))
  }

  private suspend fun downloadImage(url: String): ByteArray =
    astronomyService.downloadImage(url).awaitResponse().body()!!.bytes()
}

fun main() = createWindow("Image Viewer", ImageGallery())
