package com.example.astronomy

import javax.swing.*
import kotlinx.coroutines.*
import retrofit2.*

suspend fun getImages(count: Int): List<ByteArray> = coroutineScope {
  val infoRequest = astronomyService.getPictures(count, apiKey)
  val imageInfo = infoRequest.awaitResponse().body()!!

  val images: List<Deferred<ByteArray>> = imageInfo.map {
    async { downloadImage(it.url) }
  }

  images.awaitAll()
}

private suspend fun downloadImage(url: String): ByteArray =
  astronomyService.downloadImage(url).awaitResponse().body()!!.bytes()

class ImageGallery : ImageViewer() {
  override suspend fun update(window: JFrame) {
    val images = getImages(6)
    displayImageGallery(window, images)
  }
}

fun main() = createWindow("Image Viewer", ImageGallery())
