package com.example.astronomy

import javax.swing.*
import retrofit2.*

fun interface ImageViewerCallback<T> : Callback<T> {
  override fun onResponse(call: Call<T>, response: Response<T>)
  override fun onFailure(call: Call<T>, t: Throwable) = t.printStackTrace()
}

fun main() = SwingUtilities.invokeLater {
  val window = createImageViewerWindow()

  try {
    loadImage(window)
  } catch (error: Throwable) { // oops, that won't work!
    window.add(JLabel("Sorry, something went wrong"), JLabel.CENTER)
  }
}

fun loadImage(window: JFrame) {
  val infoRequest = astronomyService.getPictures(count = 1, apiKey)
  infoRequest.enqueue(ImageViewerCallback { _, infoResponse ->
    val imageUrl = infoResponse.body()!!.single().url
    val imageRequest = astronomyService.downloadImage(imageUrl)
    imageRequest.enqueue(ImageViewerCallback { _, imageResponse ->
      SwingUtilities.invokeLater {
        window.displayImage(imageResponse.body()!!.bytes())
      }
    })
  })
}
