package com.example.astronomy

import java.awt.event.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import retrofit2.*

class ImageViewer : WindowAdapter() {
  val coroutineScope = MainScope()

  override fun windowOpened(e: WindowEvent) {
    val window = e.window as JFrame
    window.add(JLabel("Loading your image…", JLabel.CENTER))
    coroutineScope.launch {
      while (true) {
        loadImage(window)
        delay(15.seconds)
      }
    }
  }

  override fun windowClosed(e: WindowEvent) {
    coroutineScope.cancel()
  }
}

fun main() = createWindow("Image Viewer", ImageViewer())

suspend fun loadImage(window: JFrame) {
  val infoRequest = astronomyService.getPictures(count = 1, apiKey)
  val infoResponse = infoRequest.awaitResponse()

  val imageUrl = infoResponse.body()!!.single().url
  val imageRequest = astronomyService.downloadImage(imageUrl)
  val imageResponse = imageRequest.awaitResponse()

  window.displayImage(imageResponse.body()!!.bytes())
}
