package com.example.coffee

import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*

suspend fun main() = supervisorScope {
  val beans = async { grindBeans() }
  val water = async { heatWater() }
  val milk = async { frothMilk() }

  val ingredients = try {
    awaitAll(beans, water, milk)
  } catch (error: OutOfMilkException) {
    println("Oops, looks like we're out of milk.")
    awaitAll(beans, water)
  }

  println("Brewing some delicious coffee with $ingredients")
}

suspend fun frothMilk(): String = throw OutOfMilkException()

class OutOfMilkException() : RuntimeException("There's no milk!")

suspend fun grindBeans(): String {
  delay(5.seconds)
  return "freshly ground coffee"
}

suspend fun heatWater(): String {
  delay(5.seconds)
  return "piping hot water"
}
