package com.example.gallery

import java.awt.*
import java.io.*
import java.nio.*
import java.nio.channels.*
import javax.imageio.*
import kotlin.coroutines.*

suspend fun readFile(file: File): ByteArray =
  AsynchronousFileChannel.open(file.toPath()).use { channel ->
    val buffer = ByteBuffer.allocate(channel.size().toInt())

    suspendCoroutine { continuation ->
      channel.read(buffer, 0, null, object : CompletionHandler<Int, Any?> {
        override fun completed(result: Int, attachment: Any?) =
          continuation.resume(buffer.array())

        override fun failed(exc: Throwable, attachment: Any?) =
          continuation.resumeWithException(exc)
      })
    }
  }

suspend fun readImage(file: File): Image {
  val data: ByteArray = readFile(file)
  return ImageIO.read(data.inputStream())
}
