package com.example.museum

import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*

fun describe(exhibit: MuseumObject): String = buildString {
  append('"', exhibit.title, '"', " is a ", exhibit.objectName)
  exhibit.objectDate.let { if (it.isNotEmpty()) append(" from ", it) }
  exhibit.artistDisplayName.let { if (it.isNotEmpty()) append(" by ", it) }
}

suspend fun main() {
  withTimeoutOrNull(15.seconds) {
    tourMuseum()
  }
}

suspend fun tourMuseum() {
  val catalog = museumService.listObjects()
  while (true) {
    try {
      lookAtArt(catalog.objectIDs.random())
    } catch (error: Throwable) {
      currentCoroutineContext().ensureActive()
      println("Skipping to next artwork due to $error")
    }
  }
}

suspend fun lookAtArt(artworkId: Int) {
  val artwork = museumService.getObject(artworkId)
  println(describe(artwork))
  delay(2.seconds)
}
