package com.example.museum

import java.util.concurrent.*
import okhttp3.*
import retrofit2.*
import retrofit2.converter.moshi.*
import retrofit2.http.*

data class MuseumCatalog(val objectIDs: List<Int>, val total: Int)

data class MuseumObject(
  val title: String,
  val artistDisplayName: String,
  val objectName: String,
  val objectDate: String
)

interface MuseumService {
  @GET("objects")
  suspend fun listObjects(): MuseumCatalog

  @GET("objects/{id}")
  suspend fun getObject(@Path("id") id: Int): MuseumObject
}

private val executor = Executors.newCachedThreadPool { task ->
  Thread(task).apply { isDaemon = true }
}

private fun createHttpClient() = OkHttpClient.Builder()
  .protocols(listOf(Protocol.HTTP_1_1)).dispatcher(Dispatcher(executor))

val museumService = Retrofit.Builder()
  .client(createHttpClient().build())
  .baseUrl("https://collectionapi.metmuseum.org/public/collection/v1/")
  .addConverterFactory(MoshiConverterFactory.create())
  .build().create<MuseumService>()
