package com.example.parallel

import java.security.*
import kotlin.random.*
import kotlinx.coroutines.*

fun salt() = Random.nextBytes(8)

fun hash(password: String, salt: ByteArray = salt()): String {
  val sha256 = MessageDigest.getInstance("sha256")

  var hash = salt + password.encodeToByteArray()
  repeat(1_000_000) {
    hash = sha256.digest(hash)
  }

  return salt.toHexString() + hash.toHexString()
}

const val iterations = 200_000_000

suspend fun slowHash(password: String, salt: ByteArray = salt()): String {
  val sha256 = MessageDigest.getInstance("sha256")

  var hash = salt + password.encodeToByteArray()
  repeat(iterations) { i ->
    if (i % 200_000 == 0) yield()
    hash = sha256.digest(hash)
  }

  return salt.toHexString() + hash.toHexString()
}
