package com.example.parallel

import kotlin.time.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*

val chars = ('a'..'z') + ('A'..'Z') + ('0'..'9') + '_' + '!' + '?'

val passwords = List(2_000) {
  CharArray(10) { chars.random() }.concatToString()
}

suspend fun main() = withContext(Dispatchers.Default) {
  val startTime = TimeSource.Monotonic.markNow()
  val numberOfCores = Runtime.getRuntime().availableProcessors()

  val passwordJob = async {
    passwords.take(numberOfCores) // one task for each CPU core
      .map { async { slowHash(it) } }.awaitAll()
  }

  launch {
    while (passwordJob.isActive) {
      delay(1.seconds)
      println("Time taken so far: ${startTime.elapsedNow()}")
    }
  }

  val values = passwordJob.await()
  println("Computed ${values.size} hashes in ${startTime.elapsedNow()}")
}
