package com.example.pets

import java.awt.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    try {
      val page = petService.listBreeds(limit = 5, page = i++).await()
      if (page.isNotEmpty()) {
        for (item in page) emit(item) // careful!
      } else {
        break
      }
    } catch (error: Exception) {
      println("Skipped one page due to $error")
    }
  }
}

suspend fun main() {
  breeds.map { downloadImage(it) }
    .take(1)
    .displaySlideshow()
}

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
      delay(5.seconds)
    }
  }
}
