package com.example.pets

import io.reactivex.rxjava3.core.*
import java.awt.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.reactive.*
import org.reactivestreams.*

val logoPublisher: Publisher<Image> = Flowable.defer {
  val url = "https://kotlinlang.org/docs/images/kotlin-logo.png"
  Flowable.fromCompletionStage(downloadImageAsync(url))
}

suspend fun main() {
  logoPublisher.asFlow().displaySlideshow()
}

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
      delay(5.seconds)
    }
  }
}
