package com.example.pets

import com.example.pets.Dogs.url
import java.util.concurrent.*
import okhttp3.*
import retrofit2.*
import retrofit2.Call
import retrofit2.converter.moshi.*
import retrofit2.http.*

object Cats {
  const val url = "https://api.thecatapi.com/v1/"
}

object Dogs {
  const val url = "https://api.thedogapi.com/v1/"
}

interface PetService {
  @GET("breeds")
  fun listBreeds(
    @Query("limit") limit: Int,
    @Query("page") page: Int
  ): Call<List<Breed>>

  @GET("images/{id}")
  suspend fun getImageInfo(@Path("id") id: String): ImageInfo

  @GET
  suspend fun downloadImage(@Url imageUrl: String): ResponseBody

  data class Breed(val name: String, val reference_image_id: String)
  data class ImageInfo(val id: String, val url: String)
}

private val executor = Executors.newCachedThreadPool { task ->
  Thread(task).apply { isDaemon = true }
}

private fun createHttpClient() = OkHttpClient.Builder()
  .protocols(listOf(Protocol.HTTP_1_1)).dispatcher(Dispatcher(executor))

val petService = Retrofit.Builder().baseUrl(url)
  .client(createHttpClient().build())
  .addConverterFactory(MoshiConverterFactory.create())
  .build().create<PetService>()
