package com.example.pets

import java.awt.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.*
import kotlinx.coroutines.flow.*
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    val page = petService.listBreeds(limit = 5, page = i++).await()
    if (page.isNotEmpty()) for (item in page) emit(item) else break
  }
}

fun getImages(): Flow<Image> =
  breeds.map { downloadImage(it) }

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
      delay(5.seconds)
    }
  }
}

fun <T> Flow<T>.produceInBackground(): Flow<T> = flow {
  coroutineScope {
    val channel = Channel<T>()

    launch {
      onEach { value -> channel.send(value) }
        .onCompletion { cause -> channel.close(cause) }
        .collect()
    }

    for (value in channel) emit(value)
  }
}

suspend fun main() {
  getImages()
    .produceInBackground()
    .displaySlideshow()
}
