package com.example.pets

import java.awt.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    val page = petService.listBreeds(limit = 5, page = i++).await()
    if (page.isNotEmpty()) for (item in page) emit(item) else break
  }
}

fun getImages(): Flow<Image> =
  breeds.map { downloadImage(it) }

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
      delay(5.seconds)
    }
  }
}

suspend fun main() = createWindow("Control Panel") { window ->
  window.size = Dimension(250, 100)
  val button = window.add(JButton("Click me!")) as JButton
  val clicks = flow {
    button.addActionListener { event ->
      emit(event) // error!
    }
  }
  clicks.collect {
    println("The button was clicked")
  }
}
