package com.example.pets

import java.awt.*
import java.awt.event.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.*
import kotlinx.coroutines.channels.BufferOverflow.*
import kotlinx.coroutines.flow.*
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    val page = petService.listBreeds(limit = 5, page = i++).await()
    if (page.isNotEmpty()) for (item in page) emit(item) else break
  }
}

fun getImages(): Flow<Image> =
  breeds.map { downloadImage(it) }

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
    }
  }
}

fun JButton.clickEvents(): Flow<ActionEvent> = callbackFlow {
  val listener = ActionListener { channel.trySendBlocking(it) }
  addActionListener(listener)
  awaitClose {
    removeActionListener(listener)
  }
}.conflate()

suspend fun main() = createWindow("Control Panel") { controlWindow ->
  controlWindow.size = Dimension(250, 100)
  val button = controlWindow.add(JButton("Click me!")) as JButton

  val images: Flow<Image> = getImages().withIndex().map { (index, value) ->
    value.also { println("Downloaded image $index") }
  }
  val slides = button.clickEvents().zip(images) { _, image -> image }

  launch { slides.displaySlideshow() }
  launch { slides.displaySlideshow() }
}
