package com.example.pets

import java.awt.*
import java.awt.event.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.*
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.flow.SharingStarted.Companion.Lazily
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    val page = petService.listBreeds(limit = 5, page = i++).await()
    if (page.isNotEmpty()) for (item in page) emit(item) else break
  }
}

fun getImages(): Flow<Image> =
  breeds.map { downloadImage(it) }

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

suspend fun Flow<Image>.displaySlideshow() {
  createWindow("Slideshow") { window ->
    collect { image ->
      window.contentPane.removeAll()
      window.add(JLabel(ImageIcon(image)))
      window.pack()
    }
  }
}

fun JButton.clickEvents(): Flow<ActionEvent> = callbackFlow {
  val listener = ActionListener { channel.trySendBlocking(it) }
  addActionListener(listener)
  awaitClose {
    removeActionListener(listener)
  }
}.conflate()

suspend fun main() = createWindow("Control Panel") { controlWindow ->
  controlWindow.size = Dimension(250, 100)
  val button = controlWindow.add(JButton("Click me!")) as JButton

  val slides = button.clickEvents().zip(getImages()) { _, image -> image }
    .shareIn(scope = this, started = Lazily, replay = 1)

  launch { slides.displaySlideshow() }
  delay(15.seconds)
  launch { slides.displaySlideshow() }
}
