package com.example.pets

import java.awt.*
import javax.imageio.*
import javax.swing.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import retrofit2.*

val breeds = flow {
  var i = 0
  while (true) {
    val page = petService.listBreeds(limit = 5, page = i++).await()
    if (page.isNotEmpty()) for (item in page) emit(item) else break
  }
}

suspend fun downloadImage(breed: PetService.Breed): Image {
  val info = petService.getImageInfo(breed.reference_image_id)
  val imageResponse = petService.downloadImage(info.url)
  return ImageIO.read(imageResponse.byteStream())
}

val images = breeds.filter { it.reference_image_id.isNotBlank() } // (1)
  .onEach { println("Downloading the next image…") } // (2)
  .map { downloadImage(it) } // (3)
  .take(10) // (4)


suspend fun main() = createWindow("Slideshow") { window ->
  images.collect { image ->
    window.contentPane.removeAll()
    window.add(JLabel(ImageIcon(image)))
    window.pack()

    delay(5.seconds)
  }
}
