package com.example.testing

import kotlin.time.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*

fun interface DataProvider<out T> {
  suspend fun fetchData(): T
}

var isOffline: Boolean = false

suspend fun <T> DataProvider<T>.fetchOrDefault(default: T): T {
  val timeout = if (isOffline) Duration.ZERO else 30.seconds
  val result = withTimeoutOrNull(timeout) {
    runCatching { fetchData() }
  }
  return if (result == null) default else result.getOrThrow()
}
