package com.example.testing

import com.example.testing.Observer.Event.*
import kotlin.time.*
import kotlin.time.Duration.Companion.seconds
import kotlinx.coroutines.*

fun interface DataProvider<out T> {
  suspend fun fetchData(): T
}

var isOffline: Boolean = false

suspend fun <T> DataProvider<T>.fetchOrDefault(default: T): T {
  val timeout = if (isOffline) Duration.ZERO else 30.seconds
  val result = withTimeoutOrNull(timeout) {
    runCatching { fetchData() }
  }
  return if (result == null) default else result.getOrThrow()
}

fun interface Observer {
  fun notify(event: Event)
  enum class Event { DownloadStarted, DownloadComplete }
}

fun <T> DataProvider<T>.withObserver(observer: Observer) = DataProvider {
  observer.notify(DownloadStarted)
  fetchData().also {
    observer.notify(DownloadComplete)
  }
}
